﻿namespace OverlappingTabs
{
    using System;
    using System.Windows;
    using System.Windows.Controls;
    using System.Windows.Interactivity;

    public class SetTabItemZIndexOnSelectionChangedBehavior : Behavior<DependencyObject>
    {
        private static readonly Type DependencyObjectType = typeof(SetTabItemZIndexOnSelectionChangedBehavior);

        private TabControl _tabControl;

        #region SourceObject Dependency Property
        public static readonly DependencyProperty SourceObjectProperty =
                               DependencyProperty.Register("SourceObject", typeof(TabControl), DependencyObjectType, new PropertyMetadata(null,
                                   (sender, args) => ((SetTabItemZIndexOnSelectionChangedBehavior)sender).OnSourceObjectPropertyChanged(args)));


        protected virtual void OnSourceObjectPropertyChanged(DependencyPropertyChangedEventArgs e)
        {
            if (e.OldValue != null)
            {
                var tabControl = e.OldValue as TabControl;

                if (tabControl != null)
                {
                    tabControl.SelectionChanged -= TabControlSelectionChanged;    
                }
                
            }

            if (e.NewValue != null)
            {
                var tabControl = e.NewValue as TabControl;

                if (tabControl != null)
                {
                    _tabControl = tabControl;
                    tabControl.SelectionChanged += TabControlSelectionChanged;                    
                }
            }
        }

        public TabControl SourceObject
        {
            get { return (TabControl)GetValue(SourceObjectProperty); }
            set { SetValue(SourceObjectProperty, value); }
        }
        #endregion SourceObject Dependency Property       

        protected override void OnAttached()
        {
            base.OnAttached();

            if (SourceObject != null)
            {
                _tabControl = SourceObject;
                _tabControl.SelectionChanged += TabControlSelectionChanged;
            }   
            else if (AssociatedObject != null && AssociatedObject is TabControl)
            {
                _tabControl = AssociatedObject as TabControl;
                _tabControl.SelectionChanged += TabControlSelectionChanged;                
            }            
        }

        private void TabControlSelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            
            if (_tabControl != null)
            {
                var tabControl = _tabControl;

                foreach (var tabItem in tabControl.Items)
                {
                    if (tabItem is TabItem)
                    {
                        ((TabItem)tabItem).SetValue(Canvas.ZIndexProperty, (tabItem == tabControl.SelectedItem ?
                            tabControl.Items.Count : (tabControl.Items.Count - 1) - tabControl.Items.IndexOf(tabItem)));    
                    }
                    
                }    
            }
            
        }

        protected override void OnDetaching()
        {
            if (AssociatedObject != null && AssociatedObject is TabControl)
            {
                ((TabControl)AssociatedObject).SelectionChanged -= TabControlSelectionChanged;
            }

            if (SourceObject != null)
            {
                SourceObject.SelectionChanged -= TabControlSelectionChanged;
            }   

            base.OnDetaching();
        }
    }
}
